@@ -1,10 +1,7 @@
-#ifndef LINT
-static const char rcsid[] = "$Header: /proj/cvs/prod/DHCP/dst/dst_api.c,v 1.6 2007/11/30 21:51:43 fdupont Exp $";
-#endif
-
 /*
  * Portions Copyright (c) 1995-1998 by Trusted Information Systems, Inc.
- * Portions Copyright (c) 2007 by Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (c) 2007,2009 by Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (c) 2012-2014 by Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -54,7 +51,8 @@ static const char rcsid[] = "$Header: /proj/cvs/prod/DHCP/dst/dst_api.c,v 1.6 20
 #include <sys/socket.h>
 #include <netinet/in.h>
 
-#include "minires/minires.h"
+#include "cdefs.h"
+#include "osdep.h"
 #include "arpa/nameser.h"
 
 #include "dst_internal.h"
@@ -98,7 +96,6 @@ dst_init()
 	done_init = 1;
 
 	s = getenv("DSTKEYPATH");
-	len = 0;
 	if (s) {
 		struct stat statbuf;
 
@@ -111,6 +108,10 @@ dst_init()
 		} else {
 			char *dp = (char *) malloc(len + 2);
 			int l;
+			if (dp == NULL) {
+				EREPORT(("malloc() failed for dp\n"));
+				return;
+			}
 			memcpy(dp, s, len + 1);
 			l = strlen (dp);
 			if (dp[l - 1] != '/') {
@@ -179,6 +180,11 @@ dst_s_get_key_struct(const char *name, const int alg, const u_int32_t flags,
 
 	memset(new_key, 0, sizeof(*new_key));
 	new_key->dk_key_name = strdup(name);
+	if (new_key->dk_key_name == NULL) {
+		EREPORT(("Unable to duplicate name for key"));
+		free(new_key);
+		return (NULL);
+	}
 	new_key->dk_alg = alg;
 	new_key->dk_flags = flags;
 	new_key->dk_proto = protocol;
@@ -341,13 +347,13 @@ dst_read_key(const char *in_keyname, const unsigned in_id,
 			 in_alg));
 		return (NULL);
 	}
-	if ((type && (DST_PUBLIC | DST_PRIVATE)) == 0) 
+	if ((type & (DST_PUBLIC | DST_PRIVATE)) == 0) 
 		return (NULL);
 	if (in_keyname == NULL) {
 		EREPORT(("dst_read_private_key(): Null key name passed in\n"));
 		return (NULL);
 	} else
-		strcpy(keyname, in_keyname);
+		strncpy(keyname, in_keyname, PATH_MAX);
 
 	/* before I read in the public key, check if it is allowed to sign */
 	if ((pubkey = dst_s_read_public_key(keyname, in_id, in_alg)) == NULL)
@@ -365,7 +371,7 @@ dst_read_key(const char *in_keyname, const unsigned in_id,
 					pubkey->dk_alg) == 0)
 		dg_key = dst_free_key(dg_key);
 
-	pubkey = dst_free_key(pubkey);
+	(void) dst_free_key(pubkey);
 	return (dg_key);
 }
 
@@ -441,6 +447,7 @@ dst_s_write_private_key(const DST_KEY *key)
 		if ((nn = fwrite(encoded_block, 1, len, fp)) != len) {
 			EREPORT(("dst_write_private_key(): Write failure on %s %d != %d errno=%d\n",
 				 file, out_len, nn, errno));
+			fclose(fp);
 			return (-5);
 		}
 		fclose(fp);
@@ -545,7 +552,11 @@ dst_s_read_public_key(const char *in_name, const unsigned in_id, int in_alg)
 		return (NULL);
 	}
 	/* read in the key string */
-	fgets(enckey, sizeof(enckey), fp);
+	if ((fgets(enckey, sizeof(enckey), fp) == NULL) &&
+	    (ferror(fp) != 0)) {
+		EREPORT(("dst_read_public_kety(): Error reading key\n"));
+		return (NULL);
+	}
 
 	/* If we aren't at end-of-file, something is wrong.  */
 	while ((c = getc(fp)) != EOF)
@@ -657,7 +668,7 @@ dst_dnskey_to_key(const char *in_name,
 	int alg ;
 	int start = DST_KEY_START;
 
-	if (rdata == NULL || len <= DST_KEY_ALG) /* no data */
+	if (in_name == NULL || rdata == NULL || len <= DST_KEY_ALG) /* no data */
 		return (NULL);
 	alg = (u_int8_t) rdata[DST_KEY_ALG];
 	if (!dst_check_algorithm(alg)) { /* make sure alg is available */
@@ -668,8 +679,6 @@ dst_dnskey_to_key(const char *in_name,
 	if ((key_st = dst_s_get_key_struct(in_name, alg, 0, 0, 0)) == NULL)
 		return (NULL);
 
-	if (in_name == NULL)
-		return (NULL);
 	key_st->dk_flags = dst_s_get_int16(rdata);
 	key_st->dk_proto = (u_int16_t) rdata[DST_KEY_PROT];
 	if (key_st->dk_flags & DST_EXTEND_FLAG) {
@@ -789,10 +798,12 @@ dst_buffer_to_key(const char *key_name,		/* name of the key */
 	    dkey->dk_func->from_dns_key != NULL) {
 		if (dkey->dk_func->from_dns_key(dkey, key_buf, key_len) < 0) {
 			EREPORT(("dst_buffer_to_key(): dst_buffer_to_hmac failed\n"));
-			return (dst_free_key(dkey));
+			(void) (dst_free_key(dkey));
+			return (NULL);
 		}
 		return (dkey);
 	}
+	(void) (dst_free_key(dkey));
 	return (NULL);
 }
 
@@ -897,6 +908,10 @@ dst_s_read_private_key_file(char *name, DST_KEY *pk_key, unsigned in_id,
 	if (pk_key->dk_key_name && !strcmp(pk_key->dk_key_name, name))
 		SAFE_FREE2(pk_key->dk_key_name, strlen(pk_key->dk_key_name));
 	pk_key->dk_key_name = (char *) strdup(name);
+	if (pk_key->dk_key_name == NULL) {
+		EREPORT(("Unable to duplicate name for key"));
+		goto fail;
+	}
 
 	/* allocate and fill in key structure */
 	if (pk_key->dk_func == NULL || pk_key->dk_func->from_file_fmt == NULL)
@@ -1006,9 +1021,12 @@ dst_free_key(DST_KEY *f_key)
 	else {
 		EREPORT(("dst_free_key(): Unknown key alg %d\n",
 			 f_key->dk_alg));
-		free(f_key->dk_KEY_struct);	/* SHOULD NOT happen */
 	}
 	if (f_key->dk_KEY_struct) {
+		/* 
+		 * We can't used SAFE_FREE* here as we do not know the size
+		 * of the structure, so no way to zero it.
+		 */
 		free(f_key->dk_KEY_struct);
 		f_key->dk_KEY_struct = NULL;
 	}
@@ -1065,6 +1083,10 @@ dst_random(const int mode, unsigned wanted, u_char *outran)
 	switch (mode) {
 	case DST_RAND_SEMI: 
 		bp = buff = (u_int32_t *) malloc(wanted+sizeof(u_int32_t));
+		if (bp == NULL) {
+			EREPORT(("malloc() failed for buff in function dst_random\n"));
+			return (0);
+		}
 		for (i = 0; i < wanted; i+= sizeof(u_int32_t), bp++) {
 			*bp = dst_s_quick_random(i);
 		}
